
/* **********************************
 File:AtInstruction.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 05.02.1999 18:50:15
 Comments: Part of the vIDE Project
***********************************/
package engine;

import java.util.*;

/**
 * Implements the @(... OR ... ) instruction
 * !! this executes ONLY ONE TIME PER Clock2 ! because it attaches to all 
 *  monitors, and then detaches from all, all the values are updated, 
 *  even if they were not producing the trigger. So, if they change in 
 *  the same Clock2, nothing happens anymore.
 */
class AtInstruction extends Instruction{
  Instruction block;
  Vector theEvents;
  
  AtInstruction(NameSpace ns, int delayBefore, InstructionDescription desc, Instruction block){
    super(ns, delayBefore, desc);
    this.block = block;
    theEvents = new Vector();
  }
  
  public void execute()throws InterpretTimeException, SimulationStoppedException{
    xConsole.trace(toString());
    attach();
    while(!alreadyOn())ControlThread.goSleepy();
    detach();
    xConsole.trace(toString() + "CONTINUE \\/");
    if(block != null) block.execute();
    if(next != null)next.execute();
  }
  
  //checks monitors to see wether anyone is triggered
  boolean nothingHappen(){
    for(Enumeration e = theEvents.elements() ; e.hasMoreElements() ; )
      if(((ValueChangeMonitor)e.nextElement()).isTriggd())return false;
    return true;
  }
  
  void attach(){
    for(Enumeration e = theEvents.elements() ; e.hasMoreElements() ; ){
      ValueChangeMonitor vcm = (ValueChangeMonitor)e.nextElement();
      synchronized(vcm){  //prevent desynch. between Monitor.trigger & attach
        vcm.attach((ControlThread)Thread.currentThread());
        vcm.notifyAll();
      }
    }
  }
  
  void detach(){
    for(Enumeration e = theEvents.elements() ; e.hasMoreElements() ; ){
      ValueChangeMonitor vcm = (ValueChangeMonitor)e.nextElement();
      synchronized(vcm){  //prevent desynch. between Monitor.trigger & attach
        vcm.detach();
        vcm.notifyAll();
      }
    }
  }
  
  boolean alreadyOn(){
    for(Enumeration e = theEvents.elements() ; e.hasMoreElements() ; )
      if(((ValueChangeMonitor)e.nextElement()).isTriggd())return true;
    return false;
  }
  
  void addMonitor(ValueChangeMonitor m){
    theEvents.addElement(m);
  }

}

