
/* **********************************
 File:ValueChangeMonitor.java
 Author: Power(power@kermit.cs.pub.ro)
 Cerated on May '99
 Comments: Part of the vIDE Project
***********************************/

package engine;
import java.util.*;

/**
 * This is a monitoring task that executes when any of the values
 * it watces has been modified
 */
class BunchMonitor extends Monitor
{
  Vector targets, lastValues;
  boolean isOn;
  Executable executor; //the action it must take if trgg'd

  BunchMonitor(NameSpace ns, Vector targets, Executable executor){
    super(ns);
    this.targets = targets;
    this.executor = executor;
    lastValues = new Vector();
    for (Enumeration e = targets.elements(); e.hasMoreElements(); ) {
      lastValues.addElement(null);
      ((Expression)(e.nextElement())).addMonitor(this);
    }
  }
  
  synchronized void trigger() throws InterpretTimeException{
    xConsole.debug("bunchmonitor " + toString() + ": trigger");
    boolean doTrigger = false;
    for (int i = 0; i < targets.size(); i++) {
      BitVector curValue = ((Expression)(targets.elementAt(i))).evaluate().getInt();
      if (curValue.compare((BitVector)(lastValues.elementAt(i))) != 0) {
        doTrigger = true;
        lastValues.setElementAt(curValue, i);
      }
    }
    if (doTrigger) {
      isOn = true;
      try {
        executor.execute();
      } catch(SimulationStoppedException ex) {
      }
      isOn = false;
    }
  }
  
  synchronized boolean isTriggd(){
    return isOn;
  }
  
  public String toString(){
    String result = "monitor(";
    for (Enumeration e = targets.elements(); e.hasMoreElements(); ) {
      result += ((Expression)(e.nextElement())).toString();
      if (e.hasMoreElements()) result += ",";
    }
    return result + ")";
  }  
}

