
/* **********************************
 File:Chunk.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 12.02.1999 02:41:03
 Comments: Part of the vIDE Project
***********************************/

package engine;

class Chunk {
  int start, end;
  BitVector b;
  ChunkDescription target; //runtime info
  
  Chunk(BitVector b, ChunkDescription target){
    this.b = b;
    this.target = target;
    this.start = b.msb;
    this.end = b.lsb;
  }

  Chunk(BitVector b, ChunkDescription target, int start, int end){
    this.target = target;
    this.start = start;
    this.end = end;
    this.b = b;
  }
  
  BitVector get()throws InterpretTimeException{
    return new BitVector(b, start, end);
  }

  /**
   * Returns the strength that drives this wire. 
   * @exception ClassCastException if the chunk does not reffer to a wire.
   */
  int getStrength(int bitIndex)throws ClassCastException{
    Wire w = (Wire)b;
    return (w.getBit(bitIndex) << 16) | w.strength[bitIndex];
  }

  
  void set(BitVector x, int start, int end)throws InterpretTimeException{
    b.attrib(this.start, this.end, x, start, end);
  }
  
  public String toString(){
    return target.toString();
  }
  
  synchronized void addMonitor(Monitor m){
    b.addMonitor(m);
  }
}






