
/* **********************************
File:ChunkDescription.java
Author: Alec(alecu@kermit.cs.pub.ro)
Created on 30.01.1999 00:12:33
Comments: Part of the vIDE Project
***********************************/

package engine;
import java.io.Serializable;
/**
 * A chunk is a part of a register or the whi\ole register, or a memory location
 * and is the base component of a concatenation.
 */
class ChunkDescription implements Serializable{
  
  ExpressionDescription start, end;
  boolean hasLimits; //true if start and end are meaningful
  String fqn;
    
  ChunkDescription(String fqn){
    this.fqn = fqn;
    start = end = null;
    hasLimits = false;
  }
    
  ChunkDescription(String fqn, ExpressionDescription start, 
                   ExpressionDescription end){
    this.fqn = fqn;
    this.start = start;
    this.end = end;
    hasLimits = true;
  }
  
  Chunk instantiate(NameSpace n)throws ParseException{

    BitVector b;
    Object target = n.resolveName(fqn);

    if(target == null)throw new ParseException("symbol \"" + fqn + "\" not found");
    
    try{
      b = (BitVector)target;
      if(hasLimits){
        try{
          return new Chunk(b, this, (int)start.instantiate(n).evaluate().getInt().value(), 
                           (int)end.instantiate(n).evaluate().getInt().value());
        }catch(UndefinedValueException ex){
          throw new ParseException("illegal index in part-select: " + this);
        }catch(InterpretTimeException ex){
          xConsole.dumpStack(ex);
          throw new ParseException("illegal index in part-select: " + this + " : " + ex);
        }
      }else return new Chunk(b, this);
    }catch(ClassCastException ex){
      Register r = (Register)target;
      if(hasLimits){
        try{
          if(r.isMemory()){
            if(start != end)throw new ParseException("part-select of a memory ?");
            return new Chunk(r.getData((int)start.instantiate(n).evaluate().getInt().value()), this);
          }else return new Chunk(r.getData(0), this, (int)start.instantiate(n).evaluate().getInt().value(), 
                                 (int)end.instantiate(n).evaluate().getInt().value());
        }catch(UndefinedValueException e){
          throw new ParseException("illegal index in part-select: " + this);
        }catch(InterpretTimeException e){
          throw new ParseException("illegal index in part-select: " + this + " : " + ex);
        }
      }else{
        if(r.isMemory())throw new ParseException("missing memory selector");
        return new Chunk(r.getData(0), this);
      }
    }
  }
  
  public String toString(){
    return fqn + (start != null ? "[" + start + ":" + end + "]" : "");
  }
}










