
/* **********************************
 File:ConstantDescription.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 12.02.1999 02:41:14
 Comments: Part of the vIDE Project
***********************************/

package engine;

class ConstantDescription implements ExpressionDescription{
  
  Result data;

  /**
   * useful to create a 0 constant instance
   */
  public static ConstantDescription zeroDescription = createNewSignedConstantDescription(1, true, false, 2, "0");

  public static ConstantDescription createNewStringConstantDescription(String image){
    xConsole.debug("StringConstantDescription: " + image);
    BitVector b = new BitVector(image, 256, image.length() * 8);
    return new ConstantDescription(b);
  }

  public static ConstantDescription createNewSignedConstantDescription(int size,
                                                                       boolean signed,
                                                                       boolean sign, 
                                                                       int base,
                                                                       String image){
    xConsole.debug("SignedConstantDescription :" + image + " sign: " + sign + " signed: " + signed); 
    BitVector b = new BitVector(image, base, size, signed, sign);
    return new ConstantDescription(b);
  }

  public static ConstantDescription createNewUnsignedConstantDescription(int size,
                                                                         int base,
                                                                         String image){
    xConsole.debug("UnignedConstantDescription :" + image); 
    BitVector b = new BitVector(image, base, size);
    return new ConstantDescription(b);
  }

  public static ConstantDescription createNewRealConstantDescription(String image){
    xConsole.debug("RealConstantDescription :" + image); 
    Real r = new Real(image);
    return new ConstantDescription(r);
  }
         
  /**
   *  not to be used: use the CreateNew...Description instead 
   */
  private ConstantDescription(Result data){
      //xConsole.debug("new ConstantDescription; data is: "+data);
    this.data = data;
  }
  
  public Expression instantiate(NameSpace n)throws ParseException{
      //xConsole.debug("ConstantDescription.insantiate; data is: "+data);
    return new Constant(n, data, this); //note that data is not a per-instance object
  }

  public String toString(){
    return data.toString();
  }

  public boolean isLeftValue(){
    return false;
  }
}











