
/* **********************************
 File:CountAssignMonitor.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 03.02.1999 20:28:50
 Comments: Part of the vIDE Project
***********************************/

package engine;

/**
 *  This is a monitor which waits for the rValue to change, and then takes care
 * to modify lValue at the end of the time unit speified by the delay values
 * (by posting a SimpleAssignInstruction).
 * <bold> Note: </bold>only typical delays are used here.
 */
class ContAssignMonitor extends Monitor{
  
  SelectionExpression lValue;
  Expression rValue;
  BitVector prevValue, curValue;

  /**
   * this will keep the last update instruction, so we can remove it if it becomes obsolete
   */
  SimpleAssignInstruction lastUpdate;

  Delay3 delays;
  
  ContAssignMonitor(NameSpace ns, SelectionExpression lValue,
                    Expression rValue, Delay3 delays){
    super(ns);
    this.lValue = lValue;
    this.rValue = rValue;
    this.delays = delays;

    lastUpdate = null;
    curValue = prevValue = BitVector.bX();
    rValue.addMonitor(this);
  }

  synchronized void trigger()throws InterpretTimeException{
    BitVector curValue  = rValue.evaluate().getBits();
    if(!curValue.equals(prevValue)){
      xConsole.trace("triggering continuous assignement" + lValue + " <<< " + curValue);
      int delay = Wire.computeTransitionDelayFor(prevValue, curValue, delays);
      prevValue = curValue;
      if(lastUpdate != null)lastUpdate.inhibate(); //if the last update is still pending, make it harmless
      Time.addFinisher(delay,
                       lastUpdate = new SimpleAssignInstruction(ns, lValue,
                                                   new BitVector(curValue)));
    }
  }
}




