/* **********************************
 File:ContChunk.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 12.02.1999 00:02:06
 Comments: Part of the vIDE Project
***********************************/

package engine;

/**
 * This is a chunk that continuously assings to its target.
 * Unless Chunk, it maintains the bits 
 * in an internal BitVector ( Chunk.b), 
 * and calls Wire's compute() (the target is data), in order for it to
 * determine its bits considering all its assignemens;
 */
class ContChunk extends Chunk implements Executable{
  
  BitVector data;
  byte strength;
  Executable next;

  /**
   * The timestamp of the last programmed update
   */
  long updateTime;

  /**
   * The computed data to be assigned to the target BitVector
   */
  BitVector newData;
  /**
   *The computed strengths to be assigned to target
   */
  byte []newStrengths;
  
  
  ContChunk(BitVector data, ChunkDescription target, int start, int end,
            byte strength){
    super(new BitVector(start, end), target, start, end);
    this.strength = strength;
    this.data = data;
  }

  /**
   * The implicit strength (strong1, strong0) is used here.
   */
  ContChunk(BitVector data, ChunkDescription target, int start, int end){
    this(data, target, start, end, Strength.defaultStrength);
  }
    
  synchronized void set(BitVector x, int start, int end)throws InterpretTimeException{
    xConsole.debug("ContChunk.set(" + x + ", " + start + ", " + end + "), b is:" + b +
                   " b.msb:" + b.msb + " b.lsb:" + b.lsb + " inc: " +
                   b.increment);
    b.attrib(this.start, this.end, x, start, end);
    Object [] dataRes = data.compute();
    newData = (BitVector)dataRes[0];
    if(dataRes.length > 1) //does the target implements strength-keeping ?
      newStrengths = (byte[])dataRes[1];
    int delay;

    try{
      Wire w = (Wire) data;
      delay = w.computeTransitionDelayFor(newData);
    }catch(ClassCastException ex){
      delay = 0;
    }

    updateTime = Time.oClock()+delay;
    Time.addFinisher(delay, this);
  }
   
  //executable stuff:

  public Executable add(Executable next){
    Executable old = this.next;
    this.next = next;
    return old;
  }
 
  public synchronized void execute()throws InterpretTimeException, SimulationStoppedException{
    if(!(updateTime > Time.oClock())){ //updateTime > oClock means that another update has 
                                       //superseeded this one, so skip it
      xConsole.trace(((ChunkDescription)target).fqn + " <<< " + newData);
      try{
        Wire w = (Wire)data;
        w.attrib(newData, newStrengths);
      }catch(ClassCastException cex){
        //seems that data is a mere reg
        data.attrib(newData);
      }
    }
    if(next != null)next.execute();
    next = null; //Time assumes that
  }

  /**
   * @return the strength for the specified index
   */
  public byte strength(int i){
    return strength; //for now, only this
  }
}









