
/* **********************************
File:ContChunkDescription.java
Author: Alec(alecu@kermit.cs.pub.ro)
Created on 30.01.1999 00:12:33
Comments:  Part of the vIDE Project
***********************************/

package engine;

/**
 * A continuous chunk description ( something like : "a[5:9]" or simply "a" )
 * used only for lValue in assign statemens.
 * A contChunk has two main features:
 * 1. It is registered to the data at wich is assigned, 
 *     so the target can compute its value by inspecting all the ConChunks
 *     added to it
 * 2. It maintains the last state, so the comparison at point 1
 *     is done on accurate data.
 */

class ContChunkDescription extends ChunkDescription{

  byte strength;

  ContChunkDescription(ChunkDescription c, byte strength)throws ParseException{
    super(c.fqn, c.start, c.end);
    this.strength = strength;
  }

  ContChunkDescription(ChunkDescription c)throws ParseException{
    this(c, Strength.defaultStrength);
  }
    
  Chunk instantiate(NameSpace ns)throws ParseException{
    BitVector b = null;
    //TODO: de aranjat ca sa mearga shi pentru registre (memorii)
    BitVector target = (BitVector)ns.resolveName(fqn); //daca nu e wire, aici arunca un ClassCastException

    if(target == null)throw new ParseException("symbol \"" + fqn + "\" not found");
    
    int nStart, nEnd;

    if(start == null){
      nStart = target.msb;
      nEnd = target.lsb;
    }else{
      try{
        nStart = (int)start.instantiate(ns).evaluate().getInt().value();
        nEnd = (int)end.instantiate(ns).evaluate().getInt().value();
      }catch(UndefinedValueException ex){
        throw new ParseException("illegal index in part-select: " + this);
      }catch(InterpretTimeException ex){
        xConsole.dumpStack(ex);
        throw new ParseException("illegal index in part-select: " + this + " : " + ex);
      }
    }
    ContChunk result = new ContChunk(target, this, nStart, nEnd, strength);
    target.addAssignement(result);  //registers this assignement to the target
    return result;
  }

  public String toString(){
    return fqn + ( start != null ? "[" + start + ( end != start ? ":" + end : "") + "]" : "");
  }
}
