
/* **********************************
 File:ControlThread.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 01.02.1999 11:40:22
 Comments: Part of the vIDE Project
***********************************/

package engine;

/**
 * Implements a thread of control which executes concurrently
 */

class ControlThread extends Thread{

  /**
   * true if the simulation is in course
   */
  public static volatile boolean doRun;
  
  /**
   * the instruction list to be
   * executed by this thread
   */
  Executable ins;

  /**
   * true if this thread is suspended
   */
  boolean sleepy;

  /**
   * counts the active threads
   * (those witch are not always...)
   */
  static int activeThreads;
  
  ControlThread(Executable ins, ThreadGroup tg){
    super(tg, (Runnable)null);
    this.ins = ins;
    sleepy = false;
  }
  
  ControlThread(Executable ins){
    this.ins = ins;
  }
  
  public void run(){
    try{
      activeThreads ++;
      try{
        ins.execute();
        xConsole.debug("Thread " + this + " done.");
      }catch(InterpretTimeException ex){
        xConsole.dumpStack(ex);
        xConsole.debug("InterpretTimeException caught in ControlThread " + this);
        xConsole.cout(ex.toString());
        Time.stopSimulation();
      }
      ins = null;
      activeThreads--;
      Time.removeThread(this);
    }catch(SimulationStoppedException ex){
      xConsole.debug(ex.toString() + "  " + this);
    }
    yield();
  }
  
  //unfortunately, these two cannot be synchronized
  public static void goSleepy(){
    ControlThread th = (ControlThread)Thread.currentThread();
    if(!th.sleepy){
      th.sleepy = true;
      th.suspend();
      th.sleepy = false;
    }
  }
  
  public void wakeUp(){
    if(sleepy){
      resume();
    }
  }

  public void forceStop(){
    //wakeUp();
    xConsole.debug("throwing SimulationStoppedException in thread: " + this);
    stop(new SimulationStoppedException());
  }
}











