
/* **********************************
 File:DataHolder.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 12.02.1999 02:41:26
 Comments: Part of the vIDE Project
***********************************/

package engine;

import java.util.*;

/**
 * Implements a "thing" witch can hold
 * a value whose change can be monitorized
 */

public abstract class DataHolder extends Assignable implements Result{

  /**
   *  the list of monitors
   */
  Vector monitors;

  public DataHolder(){
    monitors = new Vector();
  }
  
  void addMonitor(Monitor m){
    monitors.addElement(m);
  }
  
  int hasMonitor(Monitor m){
    return monitors.lastIndexOf(m);
  }
  
  boolean removeMonitor(Monitor m){
    return monitors.removeElement(m);
  }

  /**
   * Returns the watch monitor for this dataholder ( or null);
   */
  Monitor getWatchMonitor(){
    //since the watch monitor is crreted at the construction time of each dataholder, 
    //it must be monitor[0]
    if(monitors != null)return (Monitor) monitors.elementAt(0);
    return null;
  }
  
  /**
   *  each DataHolder should call this method 
   *  after his data has changed
   */
  void notifyMonitors()throws InterpretTimeException{
    for(Enumeration e = monitors.elements() ; e.hasMoreElements() ; )
      ((Monitor)e.nextElement()).trigger();
  }
  
}
