
/* **********************************
 File:FunctionCallExpressionDescription.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on Sun May  2 15:34:04 EDT 1999
 Comments: Part of the vIDE Project
***********************************/


package engine;

import middle.*;
import java.util.*;

/**
 * The description of a funcction call expression. 
 * It creates the required registers for the actual parameters (whose types are known only at instantiation time).
 */
class FunctionCallExpressionDescription implements ExpressionDescription{

  /**
   * Contains the expressionDescriptions for the actual parameters.
   */
  Vector actualParamsDescs;

  /**
   * The invoked func name.
   */
  String funcName;
  

  FunctionCallExpressionDescription(String funcName, Vector actualParamsDescs){
    this.actualParamsDescs = actualParamsDescs;
    this.funcName = funcName;
  }

  /**
   * Creates the placeholders for the actuel parameters transfer and the Instruction for the func invocation.
   */
  public Expression instantiate(NameSpace ns)throws ParseException{
    Vector actualParams = new Vector();
    Vector paramInstances;

    for(Enumeration e = actualParamsDescs.elements() ; e.hasMoreElements() ; ){
      //link the expressions to the caller's NameSpace (ns)
      try{
        actualParams.addElement(((ExpressionDescription)e.nextElement()).instantiate(ns));
      }catch(ParseException ex){
        xConsole.dumpStack(ex);
        throw new ParseException(super.toString() + " error:" + ex);
      }
    }

    NameSpace funcSpace;
    try{
      funcSpace = (NameSpace)ns.resolveName(funcName);
      //just checking that this name refers the correct NameSpace
      if(funcSpace == null)throw new ParseException(super.toString() + " error:symbol \"" + funcName + "\" not found");
      if(! (funcSpace.desc instanceof FunctionDescription))throw new ClassCastException();
    }catch(ClassCastException ex){
      throw new ParseException(super.toString() + " error:" + funcName + " is not a function");
    }

    FunctionDescription funcDescription = (FunctionDescription) funcSpace.desc;
    try{
      paramInstances = funcDescription.createParams(funcSpace, actualParams);
    }catch(ParseException ex){
      xConsole.dumpStack(ex);
      throw new ParseException(toString() + " error:" + ex);
    }
    xConsole.debug("func description is: " + funcDescription);
    try{
      return new FunctionCallExpression(ns, this, funcDescription, actualParams, paramInstances,
                                        funcDescription.makeInstructionInstance(funcSpace));
    }catch(ParseException ex){
      xConsole.dumpStack(ex);
      xConsole.cout(ex.toString());
      throw new ParseException("error inside called function");
    }
  }

  public String toString(){
    String res =  funcName + "(" ;
    for(Enumeration e = actualParamsDescs.elements() ; e.hasMoreElements() ;){
      res += ((ExpressionDescription)e.nextElement()).toString();
      if(e.hasMoreElements())res += ", ";
    }
    res += ");";
    return res;
  }
}




