
/* **********************************
File:GenericSelector.java
Author: Alec(alecu@kermit.cs.pub.ro)
Created on 18:04:1999 14:48:**
Comments: Part of the vIDE Project
***********************************/

package engine;

/**
 * A selector for an unknown type of register. Used for late name resolving.
 */
class GenericSelectorDescription implements ExpressionDescription,
                                            AssignableSelection{
    //combines the RealSelector, IntegerSelector

  String fqn;
  ExpressionDescription start, end;
  SelectionExpressionDescription sed;

  /**
   * true if start & end are valid.
   */
  boolean hasLimits;
  
  /**
   * true if this selector will be continuously assigned
   */
  boolean tryContSelection;

  /**
   * The type-specific selector. Created at the first instantiation
   * request, and used to make the other
   * instantiations faster.
   */
  ExpressionDescription theRealSelector;

  GenericSelectorDescription(String fqn, ExpressionDescription start,
                             ExpressionDescription end, 
                             boolean hasLimits){
    this.fqn = fqn;
    this.start = start;
    this.end = end;
    this.hasLimits = hasLimits;
    tryContSelection = false;
  }


  public Expression instantiate(NameSpace ns)throws ParseException{

    Object target = ns.resolveName(fqn);

    if(target == null)throw new ParseException("symbol \"" + fqn + "\" not found");

    try{
      Register r = (Register)target;
      if(hasLimits){
        if(r.isMemory()){
          if(tryContSelection)
            throw new ParseException("memory cannot be continuously assigned");
          if(start != end)throw new ParseException("part-select of a memory ?");
          int nIndex;
          try{
            nIndex = (int) start.instantiate(ns).evaluate().getInt().value();
          }catch(UndefinedValueException ex){
            throw new ParseException("invalid index in memory select:"
                                     + this + ": the value is not defined");
          }catch(InterpretTimeException ex){
            xConsole.dumpStack(ex);
            throw new ParseException("invalid index in memory select:"
                                     + this + ": " + ex);
          }
          //TODO: pt asignarea registrilor trebuie umblat aici------------------------
          if(tryContSelection)throw new
                                ParseException("Unimplemented: caontassign for regs");
          if(r.desc.getType() == Symbol.intType)
            return new IntSelector(ns, r.getData(nIndex), this);
          else{
            if(sed == null){
              sed = new SelectionExpressionDescription();
              sed.addChunk(new ChunkDescription(fqn));
              //Chunkdescription nu 
              //mai are nevoie de ModuleDescription
            }
            return sed.instantiate(ns);
          }
        }else{
          if(sed == null){
            sed = new SelectionExpressionDescription();
            sed.addChunk(new ChunkDescription(fqn, start, end));
            //Chunkdescription nu mai 
            //are nevoie de ModuleDescription
          }
          return sed.instantiate(ns);
        }
      }else{
        if(r.isMemory())
          throw new ParseException("missing index in memory select: "
                                   + this);
        if((r.getType() & Symbol.regType) != 0){ //could be also a wire, but still unsigned
          if(sed == null){
            sed = new SelectionExpressionDescription();
            sed.addChunk(new ChunkDescription(fqn));
            //Chunkdescription nu mai are 
            //nevoie de ModuleDescription
          }
          return sed.instantiate(ns);
        }else
          return new IntSelector(ns, r.getData(0), this); 
      }
    }catch(ClassCastException ex){}

    try{
      Real r = (Real)target;
      if(tryContSelection)
        throw new ParseException("real type cannot be continuously assigned");
      if(hasLimits)
        throw new ParseException("real " + r + " accessed at bit level");
      return new RealSelector(ns, r, this);
    }catch(ClassCastException ex){}

    try{
      Wire w = (Wire)target;
      if(sed == null){
        sed = new SelectionExpressionDescription();
        ChunkDescription ckd = hasLimits ?
          new ChunkDescription(fqn, start, end) :
          new ChunkDescription(fqn);
        if(tryContSelection)
          ckd = new ContChunkDescription(ckd);
        sed.addChunk(ckd);
      }
      return sed.instantiate(ns);
    }catch(ClassCastException ex){}

    try{
      Parameter p = (Parameter)target;
      return p;
    }catch(ClassCastException ex){}

    throw new ParseException("symbol: \"" + target + "\" is not a register !");
  }

  public void toContSelection()throws ParseException{
    tryContSelection = true;
  }

  public String toString(){
    return fqn + ( start != null ? "[" + start +
                   (end != null? ":" + end : "") + "]":"" );
  }
}


















