
/* **********************************
 File:ModuleDescription.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 12.02.1999 02:42:47
 Comments: Part of the vIDE Project
***********************************/

package engine;

import java.util.*;
import middle.*;

/**
 * This contains the description of the inside structure of a module (regs, integers, 
 * threads of control, etc).
 * Its main purpose is to creatye all the instances needed inside a module ( which in turn
 * is created by ModuleInstanceDesscription.instantiate(..)).
 */
public class ModuleDescription extends DefaultNameSpaceDescription {

  /**
   * The file that contains this module description.
   */
  String fileName;

  /**
   * The line at wich this module begins.
   */
  int line;

  ModuleDescription(String fileName, int line, String name){
    super(name);
    this.fileName = fileName;
    this.line = line;
    this.name = name;
  }
  
  boolean isTop(){
    return !(VeriParser.instantiatedModules.contains(name));
  }  
  
  
  void instantiateAll(Module m, ScopeNode thisScope)throws ParseException{
    super.instantiateAll(m, thisScope);
  }

  public void checkup() throws ParseException {
    for (Enumeration e = ports.elements() ; e.hasMoreElements() ;) {
      PortDescription p = (PortDescription)e.nextElement();
      if (regs.get(p.name) == null)
        addWire( p.name, p.lineNo, p.msbExp, p.lsbExp,
                 WireDescription.defaultExpandType,
                 WireDescription.defaultNetType,
                 Delay3Description.nullDelay);
    }    
  }

  void checkPorts(NameSpace ns) throws ParseException {
    for (Enumeration e = ports.elements() ; e.hasMoreElements() ; ) {
      PortDescription p = (PortDescription)e.nextElement();
      p.instantiate(ns, null);
      int nMsb, nLsb;
      Object o = ns.resolveNameExact(p.name);
      try {
        Wire w = (Wire) o;
        try{
          p.checkRange(w.msb, w.lsb);
        } catch (ParseException pex){
          throw new ParseException(toString(p.lineNo) + pex);
        }
        continue;
      } catch (ClassCastException cex) {}
      try {
        Register r = (Register) o;
        if(r.isMemory())
          throw new ParseException(toString(p.lineNo) + ":error : ports cannot be memories");
        BitVector b = r.getData(0);
        try {
          p.checkRange(b.msb, b.lsb);
        } catch (ParseException pex) {
          throw new ParseException(toString(p.lineNo) + pex);
        }
        continue;
      } catch (ClassCastException cex) {
        throw new ParseException(toString(p.lineNo) + ":error : bad port data type");
      }
    }

  }

  public String toString(){
    return fileName + ":" + name;
  }

  public String toString(int line){
    return fileName + ":" + line + ":" + name;
  }
  
  public String getFileName(){
    return fileName;
  }
}









