/* **********************************
 File:ModuleInstanceDescription.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 12.02.1999 02:42:51
 Comments: Part of the vIDE Project 
***********************************/

package engine;

import java.util.*;
import middle.*;

/**
 * Describes the way a module is instantiated (it describes the assignements between 
 * the container module and the submodule ports).
 */
//TODO this should be derived from ScopeInstanceDescription .
class ModuleInstanceDescription extends Symbol{
  
  /**
   * Contains the Expressions (these refer to the parent !) descriptions to be linked
   * to ports in an odered list. 
   */
  Vector portsVector;
  Hashtable portsHash;

  /**
   * Contains a list of (name, ExpressionDescription (these refer to the parent !))
   * pairs to be linked to ports by names. 
  Hashtable portsHash;

  /**
   * The descriptor of the submodule.
   */
  ModuleDescription descriptor; 

  /**
   * The name of descriptor(if forward referenced). Must be resolved at 
   * instantiation time.
   */
  String descriptorName;
  
  ModuleInstanceDescription(String name, int lineNo,
                            String descriptorName, Hashtable portsHash){
    this(name, lineNo, descriptorName, null, null, portsHash);
  }

  ModuleInstanceDescription(String name, int lineNo,
                            String descriptorName, Vector portsVector){
    this(name, lineNo, descriptorName, null, portsVector, null);
  }

  ModuleInstanceDescription(String name, int lineNo,
                            String descriptorName,
                            ModuleDescription descriptor,
                            Vector portsVector,
                            Hashtable portsHash){
    super(name, lineNo);
    this.portsVector = portsVector;
    this.portsHash = portsHash;
    this.descriptor = descriptor;
    this.descriptorName = descriptorName;
  }
  

  Object instantiate(NameSpace parent, ScopeNode scope)throws ParseException{

    if(descriptor == null) //"late" linking ?
      descriptor = (ModuleDescription)VeriParser.getModuleByName(descriptorName);
    if(descriptor == null) throw new
                             ParseException(parent.desc.toString(lineNo) +
                                            " error:undefined module: " +
                                            descriptorName);
    
    Module m = new Module(descriptor, parent, this,
                          scope.addScope(Constants.moduleIcon, name));
    if(parent.desc.name() == VeriParser.topName)return m; //top module ?
    
    //set up the links:
    if(portsVector == null){
      //'by name' connection list: order the arguments into portVector
      portsVector = new Vector();
      portsVector.setSize(portsHash.size());
      for (Enumeration e = portsHash.elements() ; e.hasMoreElements() ;) {
        ConnectionDescription cd = (ConnectionDescription) e.nextElement();
        int index =
          descriptor.connectionList.indexOf(descriptor.connectionHash.get(cd.name));
        if (index == -1) throw new ParseException(parent.desc.toString(lineNo) +
                                                  ":error : no port \"" +
                                                  cd.name + "\" found in module \"" +
                                                  m.desc.name() + "\"");
        portsVector.setElementAt(cd.portExpDesc, index);
      }
    }
    try{
    portLoop:
      for(Enumeration extern = portsVector.elements(),
            intern = descriptor.connectionList.elements(); extern.hasMoreElements() ||
            intern.hasMoreElements() ; ){
        
        ConnectionDescription cd =
          (ConnectionDescription)intern.nextElement();
        try{
          cd.computeType((DefaultNameSpaceDescription) m.desc);
        } catch (ParseException pex) {
          throw new ParseException (m.desc.toString(cd.lineNo) + pex);
        }
        ContAssignMonitor mon;
        ExpressionDescription exp =
          (ExpressionDescription)extern.nextElement();
        if(exp == null)continue portLoop; //unconnected port
        Expression connExpInstance = null, expInstance = null;

        xConsole.debug("matching " + cd + " with " + exp);

        if((cd.type & PortDescription.output) != 0)
          //prepare exp for continuous assignement
          try{
            ((AssignableSelection)exp).toContSelection();  
          }catch(ClassCastException ex){
            throw new ParseException(parent.desc.toString(lineNo) +
                                     " error:invalid argument in module instantiation");
          }

        if ((cd.type & PortDescription.input) != 0) {
          //prepare port for cont. assign.
          try{
            cd.prepareForContSelection();
          }catch(ParseException pex){
            xConsole.dumpStack(pex);
            throw new ParseException(parent.desc.toString(lineNo) + pex);
          }
          try{
            mon = new ContAssignMonitor(m, 
                                        (SelectionExpression)
                                        (connExpInstance =
                                         (Expression)cd.instantiate(m, null)),
                                        expInstance = exp.instantiate(parent),
                                        Delay3Description.nullDelay.instantiate(null));
          }catch(InterpretTimeException itex){/*from nullDelay instantiation*/}
        }

        if((cd.type & PortDescription.output) != 0){
          try{
            mon = new ContAssignMonitor(parent, 
                                        (SelectionExpression)(expInstance != null ?
                                                              expInstance : 
                                                              exp.instantiate(parent)),
                                        connExpInstance != null ? connExpInstance :
                                        (Expression)cd.instantiate(m, null),
                                        Delay3Description.nullDelay.instantiate(null));
          }catch(InterpretTimeException itex){/*from nullDelay instantiation*/}
        }

      }
    }catch(NoSuchElementException ex){
      //oops, the port lists don't match !
      xConsole.dumpStack(ex);
      throw new ParseException(parent.desc.toString(lineNo) +
                               " error : port lists don't match");
    }
    return  m;
  }

  /**
   * Used during the serialization process
   * in order to prevent other descriptions
   * reffered by  a ModuleDescription to be
   * serialized
   * @see NameSpaceDescription.breakLinks
   */
  public void breakLink() {
    descriptor = null;
  }
  
  public int getType(){
    return moduleType;
  }

  
}














