
/* **********************************
File:NameSpaceDescription.java
Author: Alec(alecu@kermit.cs.pub.ro)
Created on 18:04:1999 14:48:**
Comments:  Part of the vIDE Project
***********************************/

package engine;

import java.util.*;
import middle.*;
import java.io.Serializable;

/**
 * The description of a NameSpace.
 */

interface NameSpaceDescription extends Serializable{

  String name();

  void addParameter(String name, int lineNo)throws ParseException;

  void defParam(String name, ExpressionDescription ex)throws ParseException;

  void addPort(String name, int lineNo, int type)throws ParseException;

  void addPort(String name, int lineNo, int type,
               ExpressionDescription msb,
               ExpressionDescription lsb)throws ParseException;

  void addConnection(String name, int lineNo,
               ExpressionDescription portExp)throws ParseException;

  /**
   * Adds a submodule into this nameSpace.
   */
  void addSubModule(String moduleName, int lineNo, String descName,
                    Vector portsVector, Hashtable portsHash)throws ParseException;

  /**
   * Adds a new sub-scope within this nameSpace.
   */
  void addScope(NameSpaceDescription scope, int lineNo)
    throws ParseException;

  void addInitial(InstructionDescription ins, int lineNo)
    throws ParseException;
  
  void addAlways(InstructionDescription ins, int lineNo)
    throws ParseException;
    
  public void addFunction(String name, int lineNo)throws ParseException;
    
  void addRegister(String name, int lineNo, ExpressionDescription msb,
                   ExpressionDescription lsb, ExpressionDescription arrayStart, 
                   ExpressionDescription arrayEnd)throws ParseException;


  void addWire(String name, int lineNo, ExpressionDescription msb, 
               ExpressionDescription lsb, int expandType, int netType,
               Delay3Description delays)throws ParseException;

  void addInteger(String name, int lineNo, ExpressionDescription arrayStart,
                  ExpressionDescription arrayEnd) throws ParseException;

  void addReal(String name, int lineNo)throws ParseException;

  void addConnection(int lineNo, SelectionExpressionDescription lValue,
                     ExpressionDescription rValue, Delay3Description delays);
    
  void addTime(String name, int lineNo)throws ParseException;

  void addRealTime(String name, int lineNo)throws ParseException;

  void addEvent(String name, int lineNo)throws ParseException; //more parameters
  
  void breakLinks();  
    
  void instantiateAll(NameSpace ns, ScopeNode thisScope)throws ParseException;

  public String toString(int lineNo);
  
  public String getFileName();
}






