
/* **********************************
File:ProcAssignInstruc.java
Author: Alec(alecu@kermit.cs.pub.ro)
Created on 30.01.1999 00:52:44
Comments: Part of the vIDE Project
***********************************/

package engine;

public class ProcAssignInstruc extends Instruction{
  
  Expression rValue;
  LeftValue lValue;
  Result buffer;      //it keeps the data for delayed assignements
  int delayInside;
  boolean blocking;
  
  ProcAssignInstruc(NameSpace ns, int delayBefore, InstructionDescription desc ,
                    LeftValue lValue,
                    Expression rValue, Instruction next,
                    int delayInside, boolean blocking){
    super(ns, delayBefore, desc);
    this.delayInside = delayInside;
    this.rValue = rValue;
    this.lValue = lValue;
    this.delayInside = delayInside;
    this.blocking = blocking;
    add(next);
    buffer = null;
    
  }
  
  ProcAssignInstruc(ProcAssignInstruc original){
    super(original.ns, original.delayBefore, original.desc);
    rValue = original.rValue;
    lValue = original.lValue;
    delayInside = original.delayInside;
    blocking = original.blocking;
    buffer = null;
  }
  
  public Object clone(){
    return new ProcAssignInstruc(this);
  }
  
  public void execute()throws InterpretTimeException, SimulationStoppedException{

    long t0 = waitToRun();
    
    try{
      buffer = rValue.evaluate();
    }catch(InterpretTimeException ex){
      xConsole.dumpStack(ex);
      throw new InterpretTimeException(desc + ": error: " + ex);
    }
    
    if(delayInside != 0){
      xConsole.trace("#" + delayInside);
      t0 += delayInside;
      while(Time.oClock() < t0)
        ControlThread.goSleepy();
    }
    if(blocking){
      xConsole.trace(toString() + " <<< (" + buffer + ")");
      try{
        lValue.assign(buffer);
      }catch(InterpretTimeException ex){
        xConsole.dumpStack(ex);
        throw new InterpretTimeException(desc + ": error: " + ex);
      }
      Thread.yield();  //give monitors a time to breath

    }else{
      xConsole.trace(toString() + "(cont.)");
      Time.addFinisher(new SimpleAssignInstruction(ns, lValue, buffer));
    }
    
    xConsole.consumeStep();
    
    if(next != null)next.execute();
    
  }
  
  public String toString(){
    return desc.toString();
  }
}
