
/* **********************************
 File:ProcAssignInstrucDescription.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 28.01.1999 14:17:48
 Comments: Part of the vIDE Project
***********************************/

package engine;

/**
 * Implements the description of a procedural assignement
 * <PRE><CODE>
 * ex:
 *     #1 a #2 = b ;  //blocking
 *   or:
 *     b <= a;     //non-blocking
 * </CODE><PRE>
 */
public class ProcAssignInstrucDescription extends InstructionDescription{
  
  ExpressionDescription rValue;
  ExpressionDescription lValue;
  int delayInside;
  boolean blocking;
  
  public ProcAssignInstrucDescription(int delayBefore, int line,NameSpaceDescription nsd,
                                      ExpressionDescription lValue,
                                      ExpressionDescription rValue,
                                      int delayinside, boolean blocking){
    super(delayBefore, line, nsd);
    this.rValue = rValue;
    this.lValue = lValue;
    delayInside = delayinside;
    this.blocking = blocking;
  }
  
  Instruction instantiate(NameSpace ns)throws ParseException{

    Expression rv;
    LeftValue lv;
    try{
      try{
        lv = (LeftValue)lValue.instantiate(ns);
      }catch(ClassCastException ex){
        throw new ParseException("lValue required");
      }
      rv = rValue.instantiate(ns);
    }catch(ParseException ex){
      xConsole.dumpStack(ex);
      throw new ParseException(super.toString() + " error:" + ex);
    }

    return new ProcAssignInstruc(ns, delayBefore, this,
                                 lv, rv,
                                 next == null ? null : next.instantiate(ns),
                                 delayInside, blocking);
  }
  
  public String toString(){
    return super.toString() + lValue + " = " + rValue;
  }
}
