
/* **********************************
File:SelectionExpression.java
Author: Alec(alecu@kermit.cs.pub.ro)
Created on 30.01.1999 00:45:53
Comments: Part of the vIDE Project
***********************************/

package engine;

import java.util.*;

/**
 * @see engine.SelectionExpressionDescription
 */

class SelectionExpression extends Expression implements LeftValue{
  Vector chunks;
  int size;
    
  SelectionExpression(NameSpace ns, Vector chunkDescriptions)throws ParseException{
    super(ns);
    chunks = new Vector();
    size = 0;
    for(Enumeration e = chunkDescriptions.elements() ; e.hasMoreElements() ; ){
      Chunk ck = ((ChunkDescription)e.nextElement()).instantiate(ns);
      chunks.addElement(ck);
      size += Math.abs(ck.start - ck.end) + 1;
    }

    length = 0;
    for(Enumeration e = chunks.elements() ; e.hasMoreElements() ; ){
      Chunk ck = (Chunk)e.nextElement();
      length += Math.abs(ck.start - ck.end) + 1;
    }

  }
  
  public Result evaluate()throws InterpretTimeException{
    BitVector result = new BitVector(size - 1, 0);
    int curBit = size - 1;
    
    for(Enumeration e = chunks.elements() ; e.hasMoreElements() ; ){
      Chunk ck = (Chunk)e.nextElement();
      result.attrib(curBit, curBit-Math.abs(ck.start - ck.end), 
                    ck.b, ck.start, ck.end);
      curBit-=Math.abs(ck.start - ck.end) + 1;
    }
    return result;
  }
  
  /**
  *  Performs the assignement truncating or expanding the rValue
  *   according to the size of lVal, and thus, it DOES NOT guarantees
  *   preservation of val. Usually, val is obtained as a result of
  *   an Expression.evaluate method, so the things fit together. But...
  **/
  public void assign(Result r)throws InterpretTimeException{

    BitVector val = r.getBits(); //this actually makes a copy of r, so
                                 // the above warning is useless...

    if(length > val.n ) val.expandS(length); //fac expandare cu semn, ca sa mearga dc. am un int negativ mai 'scurt'
    if(length < val.n ) val.trunc(length);
    int curBit = val.msb;
    for(Enumeration e = chunks.elements() ; e.hasMoreElements() ; ){
      Chunk ck = (Chunk)e.nextElement();
      ck.set(val, 
        curBit, curBit - Math.abs(ck.start - ck.end) *  val.increment);
      curBit -= ( Math.abs(ck.start - ck.end) +1 ) *  val.increment;
    }
  }

  /**
   * If this is a WireSelectionExpression, it
   * returns the strength & the value of the given
   * wire bit packed inside an integer: (bit << 16) | strength.
   * If not, the result is undefined.
   */
  public int getStrength(int bitIndex){
    int firstBit = length-1, lastBit = length-1;
    xConsole.debug("SelectionExpression.getStrength(" + bitIndex + ")");
    try{
      for(Enumeration e = chunks.elements() ; e.hasMoreElements() ; ){
        Chunk ck = (Chunk)e.nextElement();
        lastBit -= Math.abs(ck.start - ck.end) +1;
        if((firstBit >= bitIndex) && (bitIndex >= lastBit)){
          int i = firstBit - bitIndex;
          if(ck.start < ck.end) i += ck.start;
          else i = ck.start - i;
          return ck.getStrength(i);
        }
        firstBit = lastBit;
      }
    }catch(ClassCastException cex){
      xConsole.dumpStack(cex);
    }
    xConsole.debug("WARNING: SelectionExpression.getStrength failed\n(if u see some ClassCastException above, the cause may be a non-cont chunk used here)");
    return 0xffffffff;
  }
  
  public String toString(){
    String result = "{";
    Enumeration e = chunks.elements();
    while(true){
      result += ((Chunk)e.nextElement());
      if(e.hasMoreElements()) result += ", ";
      else break;
    }
    result += "}";
    return result;
  }
  
  void addMonitor(Monitor m){
    for(Enumeration e = chunks.elements();e.hasMoreElements();){
      ((Chunk)e.nextElement()).addMonitor(m);
    }
  }

  /**
   * Provides a way for some external chackups
   */
  public Enumeration elements(){
    return chunks.elements();
  }

  public boolean isLeftValue(){
    return true;
  }

}



