
/* **********************************
 File:SimpleAssignInstruction.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 03.02.1999 02:46:53
 Comments: Part of the vIDE Project
***********************************/

package engine;

/**
 *  This is used to complete the non-blocking and wire assignement
 * at the end of the time unit. It's not delayed. THe value is determined at the time
 * of the non-blocking assignement exectution, and the assignement is programmed by
 * appending a SimpleAssignInstruction in the Time.finishList FIFO vector.
 */
class SimpleAssignInstruction extends Instruction{
  LeftValue lv;
  Result r;
  boolean active;
  
  SimpleAssignInstruction (NameSpace ns, LeftValue lv, Result r){
    super(ns, 0, null);
    this.r = r;
    this.lv = lv;
    active = true;
  }

  public void inhibate(){
    active = false;
  }
  
  public void execute()throws  InterpretTimeException, SimulationStoppedException{
    
    if(active){
      if(!xConsole.enableExecution()){
        xConsole.debug("exit received");
        Time.stopSimulation();
        throw new SimulationStoppedException();
      }
      
      try{
        lv.assign(r);
        Thread.yield();  //give monitors a time to breath
      }catch(InterpretTimeException ex){
        xConsole.dumpStack(ex);
        xConsole.cout("Size does not match in attrib. :" + toString());
        return;
      }
      xConsole.trace(toString());
      
      xConsole.consumeStep();
    } 

    if(next != null) next.execute();
  }
  
  public String toString(){
    return lv + " <= " + r;
  }
}



