
/* **********************************
 File:Time.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Creted on 12.02.1999 02:44:11
 Comments:  Part of the vIDE Project
***********************************/

package engine;

import java.util.*;

/**
 * The main thread of the simulation & the time manager.
 */
class Time {
  
  static long time;
  static int time_2; //0 if we are in the firsth part of the time unit
                         //1 if it is the secont part ( i.e. the one when cont.
                         //assignements take part)
  
  static Vector controlThreads = new Vector();

  /**
   * The stuff pending to the end of the time units, time - ordered ;
   */
  static Vector lastFinisher = new Vector(), finishList = new Vector(); 

  /**
   * Contains a list of instructions to execute after each time unit
   */
  static Vector afterFinish = new Vector();
  static boolean anyThreads;

  /**
   * make it false to stop the simulation
   */
  static boolean keepWorking;

    /**
     * starts all the threads
     */
  static void  startSimulation(){

    time = 0;
    time_2 = 0;

    ControlThread.doRun = true;
    ControlThread.activeThreads = 0;

    anyThreads = false;
    keepWorking = true;

    lastFinisher.clear();
    finishList.clear();

    xConsole.trace("SIMULATION TIME IS 0");
    //start the threads:
    synchronized(controlThreads){
      for(Enumeration e = controlThreads.elements() ; e.hasMoreElements() ; ){
        ((Thread)e.nextElement()).start();
        anyThreads = true;
      }
    }
  }
  
  public static synchronized long oClock(){
    return time;
  }
  
  public static long clk2(){
    return (time << 2) + time_2;  //this returns the number of half time units.
  }

  //append an instruction for the end of the current time unit
  public static void addFinisher(int delay, Executable i){
    xConsole.debug("adding finisher for #" + (delay + time) + " : " + i);
    if(finishList.size() < delay+10){
      //add some extra space to reduce the time wasted with copying stuff
      finishList.setSize(delay+10);
      lastFinisher.setSize(delay+10);
    }

    if(lastFinisher.elementAt(delay) != null)((Executable)lastFinisher.elementAt(delay)).add(i);
    else finishList.setElementAt(i, delay);
    lastFinisher.setElementAt(i, delay);
  }

  public static void addFinisher(Instruction i){
    addFinisher(0, i);
  }

  public static void addAfterFinish(Executable i){
    afterFinish.addElement(i);
  }
  
  static void removeThread(Thread t){
    synchronized(controlThreads){
      controlThreads.removeElement(t);
    }
    xConsole.debug("removed thread: " + t);
  }

  static void stopSimulation(){
    synchronized(controlThreads){

      ControlThread.doRun = false;

      //stop all the threads, no matter what they were doing !
      for(Enumeration e = controlThreads.elements() ; e.hasMoreElements() ; ){
        ControlThread ct = (ControlThread)e.nextElement();
        xConsole.debug("attempting to stop thread : " + ct);
        if(ct != Thread.currentThread())ct.forceStop(); //wakeUp must check wheter it still worths running
      }

      xConsole.debug("done");
      keepWorking = false;
      controlThreads.clear();
      lastFinisher.clear();
      finishList.clear();
      afterFinish.clear();
    }
  }
  
  /**
   * the main simulation loop
   */
  static void runSimulation(){
    boolean allSleepy = true;
    //prevents waiting for eny thread to become active if
    //there isn't one
    if(!anyThreads){
      xConsole.cout("nothing to do!");
      return;
    }

    //wait for some threads to start their jobs
    while(ControlThread.activeThreads == 0)
      Thread.yield();

    //the main loop
    while(keepWorking){
      try{
        Thread.sleep(100);
      }catch(InterruptedException ex){
        xConsole.debug("Who dares to disturbe the TIME ?");
      }

      synchronized(controlThreads){
        if(ControlThread.activeThreads == 0){
          xConsole.cout("Nothing left to do; exiting");
          stopSimulation();
          return;
        }
        allSleepy = true;
        for(Enumeration e = controlThreads.elements() ; e.hasMoreElements() ; ){
          ControlThread t = (ControlThread)e.nextElement();
          synchronized(t){
            allSleepy = allSleepy && t.sleepy;
          }
        }
      }
      if(allSleepy){
        time_2 = 1;
        if(finishList.size() > 0 &&finishList.firstElement() != null){ //perform pending assignements first; this may 
                                               //wake up some threads
          xConsole.debug("finishing for time #" + time);
          try{
            ((Executable)finishList.firstElement()).execute();
          }catch(InterpretTimeException ex){
            xConsole.dumpStack(ex);
            xConsole.cout(ex.toString());
            stopSimulation();
            return;
          }catch(SimulationStoppedException ex){
            stopSimulation();
            return;
          }
          //remove the executed part:
          finishList.setElementAt(null, 0);
          lastFinisher.setElementAt(null, 0);
        }else{
          //first clean up the finish list
          if(finishList.size() > 0){
            finishList.removeElementAt(0);
            lastFinisher.removeElementAt(0);
          }

          try{
            for(Enumeration e = afterFinish.elements() ; e.hasMoreElements() ;)
              ((Executable)e.nextElement()).execute();
          }catch(InterpretTimeException ex){
            xConsole.dumpStack(ex);
            xConsole.cout(ex.toString());
            stopSimulation();
            return;
          }catch(SimulationStoppedException ex){
            stopSimulation();
            return;
          }
          xConsole.trace("SIMULATION TIME IS " + ++time);
          time_2 = 0;
          System.gc() ; //make room for new action !
          xConsole.consumeUnit();
          synchronized(controlThreads){
            for(Enumeration e = controlThreads.elements() ; e.hasMoreElements() ; )
              ((ControlThread)e.nextElement()).wakeUp();  //c'm on man, wake up !
          }
        }
      }
    }
  }
}






