
/* **********************************
 File:ValueChangeMonitor.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 09.02.1999 02:00:30
 Comments: Part of the vIDE Project
***********************************/

package engine;

/**
 * It's is a monitor that watches for the change of an expression, 
 * and then wakes up the associated thread (and, now and then, 
 * it gets stuck, i don't jnow why...)
 */

class ValueChangeMonitor extends Monitor
{
  Expression target;
  BitVector lastValue, curValue;
  long lastTime;
  ControlThread waiter;
  boolean isOn;
  
  static final int  EVT_CHANGE  = 0x00000001,
                    EVT_POSEDGE = 0x00000010,
                    EVT_NEGEDGE = 0x00000100;
  
  int mask;
  
  ValueChangeMonitor(NameSpace ns, Expression target, int mask){
    super(ns);
    this.target = target;
    this.mask = mask;
    lastValue = null;
    target.addMonitor(this);
  }
  
  synchronized void trigger()throws InterpretTimeException{
    //    xConsole.debug("trigger");
    BitVector curValue = target.evaluate().getInt();
    if(((curValue.compare(lastValue) & mask) != 0) &&
      ( Time.clk2() > lastTime) ){
      isOn = true;
      if(waiter!=null){   //if waiter==null, the instruction has not yet performed attach, so
                          //neither alreadOn() ; no need to wake it up
        while(!waiter.sleepy) //if attach() completed, goSleepy() will follow soon;
                              //but beware of resume it too quick ! (before goSleepy())
          if(waiter==null)return;  //if this is true, thread was detached, so give up.
                                   //(probabily another monitor was more lucky)
        waiter.resume();
      }
    }
  }
  
  synchronized void attach(ControlThread waiter){
    this.waiter = waiter;
  }
  
  void detach(){
    waiter = null;
    isOn = false;
    lastValue = curValue;
    lastTime = Time.clk2();
  }
  
  synchronized boolean isTriggd(){
    return isOn;
  }
  
  public String toString(){
    String result = " ";
    if((mask & EVT_POSEDGE) != 0) result = "posedge ";
    else if((mask & EVT_NEGEDGE) != 0) result = "negedge ";
    result += target;
    return result;
  }
  
}

