/* **********************************
 File:Wire.java
 Author: Alec(alecu@kermit.cs.pub.ro)
 Created on 12.02.1999 02:44:55
 Comments: Part of the vIDE Project
***********************************/

package engine;

import java.util.*;

/**
 * This is the class for the <code>wire</code> construct and all other
 * nets except trireg (@see Trireg).
 * @see WireDescription.
 */

class Wire extends BitVector{
  
  NameSpace ns;
  int expandType;
  Delay3 delays;
  WireDescription desc;  //runtime type information
  /**
   * This makes the difference between wire, wand & stuff
   */
  int truthTable[][];

  /**
   * here the strength will be kept.
   */
  byte []strength;
  int type;
    
  Wire(NameSpace ns, WireDescription desc, int type,
       int nStart, int nEnd, int expandType, Delay3 delays){
    super(nStart, nEnd);
    this.ns = ns;
    this.desc = desc;
    this.expandType = expandType;
    this.delays = delays;
    strength = new byte[n];
    this.type = type;
    initToDefaults(this, strength);
    truthTable = lookupTables[type]; //see Assigneble for available types
  }
 
  public int getType(){
    return Symbol.wireType | Symbol.regType;
  }

  public int computeTransitionDelayFor(BitVector b){
    return computeTransitionDelayFor(this, b, delays);
  }

  /**
   * Returns the delay for the transition from the current
   * value (before) to a given one (after).
   */
  public static int computeTransitionDelayFor(BitVector before,
                                              BitVector after,
                                              Delay3 delays){
    
    if(before.equals(after)){
      return 0;
    }
    
    switch(delays.delaySpecs){
    case 1:
      return delays.delay1Typ;
    case 2:
      if(!after.isX()){
        if(after.isZ())return Math.min(delays.delay1Typ, delays.delay2Typ);
        if(after.isNull())
          return delays.delay2Typ;    // !0 -> 0
        return delays.delay1Typ;       //0 -> !0
      }else
        return Math.min(delays.delay1Typ, delays.delay2Typ);   // ?? -> X
    case 3:
      if(!after.isX()){
        if(after.isZ())return delays.delay3Typ; // ?? -> Z
        if(after.isNull())
          return delays.delay2Typ;    // !0 -> 0
        return delays.delay1Typ;       //0 -> !0
      }else
        return Math.min(delays.delay3Typ,
                        Math.min(delays.delay1Typ, delays.delay2Typ));   // ?? -> X
    }

    throw new Error("uncovered case in Wire.computeTransitionDelayFor: " + before + " -> " + after);
  }

  /**
   * The improved version here is strength - aware
   */
  synchronized Object[] compute() throws InterpretTimeException{
    BitVector b = new BitVector(this);
    byte strengths[] = new byte[b.n];
    initToDefaults(b, strengths);
    xConsole.debug("Wire.compute->");
    for(Enumeration e = data.elements() ; e.hasMoreElements() ; ){
      ContChunk ck = (ContChunk)e.nextElement();
      xConsole.debug("chunk: " + ck);
      for(int i = ck.start, inc = ck.start <= ck.end ? 1 : -1
            ;; i += inc){
        xConsole.debug("i = " + i);
        int index = increment == 1 ? i - lsb : lsb - i;
        int result = Strength.reconcileDrivers(truthTable,
                                               strength[index],
                                               ck.strength(i),
                                               b.getBit(i),
                                               ck.b.getBit(i));
        b.setBit(i, result >> 16);
        strength[index] = (byte) (result & 0xff);
        if(i == ck.end)break;
      }
    }
    xConsole.debug("<-Wire.compute");
    Object [] res = {b, strengths};
    return res;
  }

  /**
   * initializes this bitvector and strength 
   * vector to the defaults for this type of net
   */
  void initToDefaults(BitVector b, byte[] strengths){
    int value = Assignable.initValDefaults[type];
    byte strength = Strength.initStrengthDefaults[type];
    for(int i = 0; i < b.n ; i++){
      b.setb(i, value);
      strengths[i] = strength;
    }
  }

  /**
   * A Strengths - aware version of attrib
   * Note that it assumes that valueas are obbtained from a 
   * compue, and so they are correct - no checks are made here.
   */
  void attrib(BitVector b, byte[] strengths)throws
  InterpretTimeException{
    strength = strengths;
    attrib(b);
  }

}













